"""
Lifted from Django 1.1.1 for loco-directory

This module collects helper functions and classes that "span" multiple levels
of MVC. In other words, these functions/classes introduce controlled coupling
for convenience's sake.
"""

from django.http import HttpResponseRedirect, HttpResponsePermanentRedirect
from django.core import urlresolvers

def redirect(to, *args, **kwargs):
    """
    Returns an HttpResponseRedirect to the apropriate URL for the arguments
    passed.
    
    The arguments could be:
    
        * A model: the model's `get_absolute_url()` function will be called.
    
        * A view name, possibly with arguments: `urlresolvers.reverse()` will
          be used to reverse-resolve the name.
         
        * A URL, which will be used as-is for the redirect location.
        
    By default issues a temporary redirect; pass permanent=True to issue a
    permanent redirect
    """
    if kwargs.pop('permanent', False):
        redirect_class = HttpResponsePermanentRedirect
    else:
        redirect_class = HttpResponseRedirect
    
    # If it's a model, use get_absolute_url()
    if hasattr(to, 'get_absolute_url'):
        return redirect_class(to.get_absolute_url())
    
    # Next try a reverse URL resolution.
    try:
        return redirect_class(urlresolvers.reverse(to, args=args, kwargs=kwargs))
    except urlresolvers.NoReverseMatch:
        # If this doesn't "feel" like a URL, re-raise.
        if '/' not in to and '.' not in to:
            raise
        
    # Finally, fall back and assume it's a URL
    return redirect_class(to)


def queryset_sum(field, qs):
    """
    Returns the summation of the values in the given field of the queryset
    Work around for missing QuerySet.aggregate() in Django 1.0
    """
    sum = 0
    for r in qs:
        if hasattr(r, field):
            f = getattr(r, field)
            if f is not None and isinstance(f, (int, float, long)):
                sum = sum + f
            else:
                raise TypeError
        else:
            raise AttributeError
    return sum

def queryset_count(field, qs):
    """
    Returns the number of non-null the values in the given field of the queryset
    Work around for missing QuerySet.aggregate() in Django 1.0
    """
    count = 0
    for r in qs:
        if hasattr(r, field):
            f = getattr(r, field)
            if f is not None:
                count = count + 1
            else:
                raise TypeError
        else:
            raise AttributeError
    return count
