# -*- coding: utf-8 -*-

from django import forms
from django.utils.translation import ugettext as _
from django.core.urlresolvers import reverse

from models import BaseEvent, GlobalEvent, TeamEvent, Attendee, TeamEventComment
from venues.models import Venue

def validate_tag(tag):
    if tag.startswith('#'):
        tag = tag[1:]
    for delim in [ " ", ",", "#" ]:
        if delim in tag:
            raise forms.ValidationError(_(u'Please just set one tag.'))
    return tag

class BaseEventForm(forms.ModelForm):
    """ 
    a form to create/update a BaseEvent
    """
    class Meta:
        model = BaseEvent
        exclude = ('date_created')

    class Media:
        css = {'all': (
                       '/media/jquery-ui/css/css/smoothness/jquery-ui.css',
                       '/media/jquery-ui-timepicker/css/ui-lightness/ui.timepickr.css',
                       )}
        js = (
              '/media/jquery/jquery.js',
              '/media/jquery-ui/jquery-ui.js',
              '/media/jquery-ui-timepicker/js/jquery.timepickr.min.js',
              '/media/jquery-ui-timepicker/js/ui.timepickr.min.js',
              '/media/js/events-ui.js',
              )
        
    def __init__(self, *args, **kargs):
        super(BaseEventForm, self).__init__(*args, **kargs)
        self.fields['date_begin'].widget = forms.SplitDateTimeWidget()
        self.fields['date_end'].widget = forms.SplitDateTimeWidget()
        
    def clean(self):
        begin = self.cleaned_data.get('date_begin')
        end = self.cleaned_data.get('date_end')
        if begin and end and begin > end:
            raise forms.ValidationError("Events can not end before they start.")
        return self.cleaned_data

class TeamEventForm(BaseEventForm):
    """ 
    a form to create/update a TeamEvent
    """
    class Meta(BaseEventForm.Meta):
        model = TeamEvent
        exclude = ('teams', 'date_created')
        
    def __init__(self, *args, **kargs):
        super(TeamEventForm, self).__init__(*args, **kargs)
        self.fields['venue'].choices = self.grouped_venue_list()
        from common.widgets import PopupRelatedFieldWidgetWrapper
        self.fields['venue'].widget = PopupRelatedFieldWidgetWrapper(self.fields['venue'].widget, reverse('venue-new'))
        
    def grouped_venue_list(self):
        """
        Returns a list of venues grouped by country
        in a way that Django will convert to OptGroups
        """
        venue_choices = [('', '---------')]
        country_choices = []
        current_country = ''
        # Ordered by country so we can tell when we're done with one
        # and starting another
        for venue in Venue.objects.all().order_by('country'):
            # If this venue has a different country, start a new group
            if venue.country != current_country:
                # Add the old group to the list if it's non-empty
                if len(country_choices) > 0:
                    venue_choices.append((current_country, country_choices))
                country_choices = []
                current_country = venue.country
            # Add venue to the current country group
            country_choices.append((venue.id, '%s (%s)'%(venue.name, venue.city)))
        
        # Add the last country group to the list if it's non-empty
        if len(country_choices) > 0:
            venue_choices.append((current_country, country_choices))
                
        return venue_choices
        
        
class GlobalEventForm(BaseEventForm):
    """ 
    a form to create/update a GlobalEvent
    """
    class Meta(BaseEventForm.Meta):
        model = GlobalEvent

    def clean(self):        
        self.cleaned_data['microbloghashtag'] = validate_tag(self.cleaned_data['microbloghashtag'])
        self.cleaned_data['pictag'] = validate_tag(self.cleaned_data['pictag'])
        return self.cleaned_data

class AttendeeRegistrationForm(forms.ModelForm):
    """
    a form to create/update an Attendee object
    """
    def __init__(self, *args, **kargs):
        super(AttendeeRegistrationForm, self).__init__(*args, **kargs)
        self.fields['guests'].min_value = 0

    class Meta:
        model = Attendee
        exclude = ('attendee', 'team_event')

class TeamEventCommentForm(forms.ModelForm):
    """
    a form to comment a team event
    """
    class Meta:
        model = TeamEventComment
        exclude = ('team_event', 'date_created', 'comment_by')

class EventSearchForm(forms.Form):
    """
    A Search form for events
    """
    q = forms.CharField(label=_('Search'), required=False, max_length=100)

    def as_line(self):
        "Returns this form rendered. Only the fields. Nothing else HTML."
        return self._html_output(u'%(label)s %(errors)s%(field)s%(help_text)s', u'%s', '', u'%s', False)


