from django.db import models
from django.contrib.auth.models import User
from teams.models import *
from venues.models import Venue
from django.utils.translation import ugettext_lazy as _
from common.shortcuts import queryset_sum, queryset_count

ATTENDEE_PROMISE_CHOICES = (
    ('sure', _('attending')),
    ('maybe', _('might be attending')),
    ('not', _('not attending')),
)

import datetime

class BaseEvent(models.Model):
    """
    a simple basic event
    """
    name = models.CharField(max_length=150, verbose_name=_('Name of the Event'))
    date_begin = models.DateTimeField(help_text=_('start date and local time of the event'), verbose_name=_('Begins'), db_index=True)
    date_end = models.DateTimeField(help_text=_('end date and local time of the event'), verbose_name=_('Ends'), db_index=True)
    description = models.TextField(help_text=_('Description of the Event'), blank=True, null=True)
    announce = models.URLField(help_text=_('URL to the announcement'), max_length=200, verify_exists=False, blank=True, null=True)
    date_created = models.DateTimeField(help_text=_('the date and time when the event was created'), default=datetime.datetime.now(), db_index=True)

    def __unicode__(self):
        return self.name
    
    def as_ical(self):
        """
        return a event as ical
        """
        dtstart = str(self.date_begin).replace(' ','T', 1).replace(':','',2).replace('-','',2)
        dtend = str(self.date_end).replace(' ','T', 1).replace(':','',2).replace('-','',2)
        return '''BEGIN:VEVENT
UID:%(id)s
DTSTART:%(dtstart)sZ
DTEND:%(dtend)sZ
CATEGORIES:Ubuntu Loco Team Event
SUMMARY:%(eventname)s
DESCRIPTION:%(description)s
END:VEVENT
''' % {'id':self.id, 'dtstart':dtstart, 'dtend':dtend, 'eventname':self.name, 'description': self.description}


class GlobalEventManager(models.Manager):
    """ manager for a global event """
    def next_events(self):
        """ a list with all upcoming global events """
        return self.filter(date_end__gt=datetime.datetime.now()).order_by('-date_begin')
    
    def history_events(self):
        """ all  global events in history """
        return self.filter(date_end__lt=datetime.datetime.now()).order_by('-date_begin')

    def random_5(self):
        """ return 5 random Team Events for this global event """
        return TeamEvent.objects.filter(global_event__id__exact=self.id).order_by('?')[:5]

    def last_5(self):
        """ return the last 5 active Team events for this global event"""
        return TeamEvent.objects.filter(global_event__id__exact=self.id).order_by('?')[5:]

class GlobalEvent(BaseEvent):
    """
    a global event. other events can attend to a global event
    """
    microbloghashtag = models.CharField(max_length=50, help_text=_('Hash Tag used for microbloggers to mention this event'), verbose_name=_('Microblogging Hash Tag'), null=True, blank=True)
    pictag = models.CharField(max_length=50, help_text=_('tag used in web albums like; picasa, flickr, and more'), verbose_name=_('Picture Tag'),null=True, blank=True)
    
    def __unicode__(self):
        return "%s %s" % (self.name, self.date_begin or "")
        
    @models.permalink
    def get_absolute_url(self):
        return ('global-event-detail', [str(self.id)])
    
    #add manager
    objects = GlobalEventManager()


class TeamEventManager(models.Manager):
    """ manager for a team event """
    def next_5_events(self):
        """ a list with the next 5 events """
# XXX: Workaround for https://bugs.launchpad.net/loco-directory/+bug/531970
        return self.filter(date_end__gt=datetime.datetime.now()-datetime.timedelta(days=1)).order_by('date_begin')[:5]

    def next_events(self):
        """ a list with all upcoming team events """
# XXX: Workaround for https://bugs.launchpad.net/loco-directory/+bug/531970
        return self.filter(date_end__gt=datetime.datetime.now()-datetime.timedelta(days=1)).order_by('date_begin')

    def history_events(self):
        """ all team events in history """
        return self.filter(date_end__lt=datetime.datetime.now()).order_by('-date_end')

    def attending(self):
        return Attendee.objects.filter(event__id__exact=self.id)

    def random_5_attendees(self):
        return Attendee.objects.filter(event__id__exact=self.id).order_by('?')[5:]

class TeamEvent(BaseEvent):
    """
    a event of one or more teams
    """
    teams = models.ManyToManyField(Team, help_text=_('teams of this event.'), verbose_name=_('Teams Participating'), db_index=True)
    global_event = models.ForeignKey(GlobalEvent, help_text=_('this team event is part of the selected global event'), verbose_name=_('Related Global Event'), blank=True, null=True)
    venue = models.ForeignKey(Venue, help_text=_('Venue of the Event'), verbose_name=_('Venue of the Event'), blank=True, null=True) 
    def __unicode__(self):
        return "%s %s %s" % (self.name, self.venue, self.date_begin or "")

    @models.permalink
    def get_absolute_url(self):
        return ('team-event-detail', [str(self.id)])

    def is_attending(self, user):
        if user.is_authenticated():
            return self.attendee_set.filter(attendee=user).count() > 0
        else:
            return False

    def total_attending(self):
        total_attending_guests = queryset_sum('guests', Attendee.objects.filter(team_event__id__exact=self.id).filter(promise="sure"))
        total_attending_registered = queryset_count('attendee', Attendee.objects.filter(team_event__id__exact=self.id).filter(promise="sure"))
        return  total_attending_guests + total_attending_registered

    def total_maybe_attending(self):

        total_maybe_attending_guests = queryset_sum('guests', Attendee.objects.filter(team_event__id__exact=self.id).filter(promise="maybe"))
        total_maybe_attending_registered = queryset_count('attendee', Attendee.objects.filter(team_event__id__exact=self.id).filter(promise="maybe"))
        return  total_maybe_attending_guests + total_maybe_attending_registered

    #add manager
    objects = TeamEventManager()


class TeamEventComment(models.Model):
    """
    a model for comments about a team event
    """
    team_event = models.ForeignKey(TeamEvent, help_text=_('the Team Event'))
    date_created = models.DateTimeField(help_text=_('the date and time when the event comment was created'), default=datetime.datetime.now(), db_index=True)
    comment_by = models.ForeignKey(User) 
    comment = models.TextField(help_text=_('Comment this Event'), db_index=True)
    
    def __unicode__(self):
        return "Team Event: %s . Comment: %s" % (self.team_event.name, self.comment)
    


class Attendee(models.Model):
    """
    a event attendee
    """
    class Meta:
        unique_together = ('team_event', 'attendee')
        
    team_event = models.ForeignKey(TeamEvent, help_text=_('the Team Event'))
    attendee = models.ForeignKey(User, help_text=_('the name of the attendee'))
    promise = models.CharField(verbose_name=_('Attending Status'), max_length=50, choices=ATTENDEE_PROMISE_CHOICES)
    guests = models.PositiveSmallIntegerField(verbose_name=_('Additional Guests'))
    
    def __unicode__(self):
        return self.attendee


