from django.template import RequestContext
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response
from django.shortcuts import get_object_or_404
from django.contrib.auth.decorators import login_required
from django.utils.translation import ugettext as _

from events.models import TeamEvent
from events.models import GlobalEvent
from teams.models import Team
from events.models import Attendee

from forms import TeamEventForm
from forms import TeamEventCommentForm
from forms import GlobalEventForm
from forms import AttendeeRegistrationForm
from forms import EventSearchForm
from django.db.models import Q

from common.utils import redirect
from common import launchpad

import datetime

def event_list(request):
    """
    a list with all events (teamevents and globalevents)
    """
    team_event_list = TeamEvent.objects.next_events()
    global_event_list = GlobalEvent.objects.next_events()
    form = EventSearchForm(data=request.GET)
    if form.is_valid():
        if form.cleaned_data['q']:
            q = form.cleaned_data['q']
            global_event_list = global_event_list.filter(Q(name__icontains=q))
            team_event_list = team_event_list.filter(Q(name__icontains=q) | Q(teams__name__icontains=q) | Q(venue__name__icontains=q) | Q(venue__city__icontains=q) | Q(venue__country__name__icontains=q) | Q(global_event__name__icontains=q))
    is_on_lc = launchpad.is_user_on_loco_council(request.user)
    context = {
        'team_event_list': team_event_list,
        'global_event_list': global_event_list,
	'user_is_on_lc': is_on_lc,
	'form': form,
    }
    return render_to_response('events/event_list.html', context,
                  RequestContext(request))

def event_list_ical(events, name):
    """Return any list events as an ical"""
    filename = "%s.ics" % name.replace(' ', '-').lower()
    response = HttpResponse(mimetype='text/calendar')
    response['Content-Disposition'] = 'attachment; filename=%s' % filename.encode('ascii', 'replace')
    response.write('''BEGIN:VCALENDAR
PRODID:-//loco.ubuntu.com//EN
VERSION:2.0
CALSCALE:GREGORIAN
METHOD:PUBLISH
X-WR-TIMEZONE:UTC
''')
    response.write('X-WR-CALNAME:%s\n' % name)
    response.write('X-WR-CALDESC:%s\n' % name)
    for event in events:
        response.write(event.as_ical())
    response.write('''END:VCALENDAR''')
    return response

def team_event_list_ical(request, team_slug):
    """
    Return a ical list with a single team events in ical format.
    """
    team = get_object_or_404(Team, lp_name=team_slug)
    team_events = TeamEvent.objects.filter(teams__lp_name=team_slug)
    return event_list_ical(team_events, team.name)

def teams_event_list_ical(request):
    """
    Return a ical list with team events in ical format.
    """
    return event_list_ical(TeamEvent.objects.all(), 'All Ubuntu LoCo Team Events')

def global_event_list_ical(request):
    """
    Return a ical list with global events in ical format.
    """
    return event_list_ical(GlobalEvent.objects.all(), 'Ubuntu Global Events')

def event_history_list(request):
    """
    a list with all historical events (teamevents and globalevents)
    """
    team_event_list = TeamEvent.objects.history_events()
    global_event_list = GlobalEvent.objects.history_events()
    context = {
        'team_event_list': team_event_list,
        'global_event_list': global_event_list,
    }
    return render_to_response('events/event_history_list.html', context,
                  RequestContext(request))


#################################################################
# Team Events
#################################################################

def team_event_detail(request, team_event_id):
    """
    detailed view for a team event
    """
    team_event_object = get_object_or_404(TeamEvent, pk=team_event_id)
    is_member = False
    for team_object in team_event_object.teams.all():
        is_member = is_member or launchpad.is_team_member(request.user, team_object)

    context = {
        'team_event_object': team_event_object,
        'user_is_attending': team_event_object.is_attending(request.user),
        'user_is_team_member': is_member,
    }
    return render_to_response('events/team_event_detail.html', 
                                         context, RequestContext(request))

@login_required
def team_event_delete(request, team_event_id):
    """
    delete a event
    """
    team_event_object = get_object_or_404(TeamEvent, pk=team_event_id)
    
    is_on_lc = launchpad.is_user_on_loco_council(request.user)
    #check if user is admin or owner of a team
    is_contact = False
    for team in team_event_object.teams.all():
        if launchpad.is_admin_or_owner(request.user.username, team):
            is_contact = True
            break

    if is_on_lc or is_contact:
        if request.method == 'POST':		
            team_event_object.delete()
            request.user.message_set.create(message=_('Team Event removed.'))
            return redirect( 'event-list' )
        else:
            context = {'team_event_object': team_event_object}
            return render_to_response('events/team_event_delete_confirm.html', context, RequestContext(request))

    else:
        request.user.message_set.create(message=_('You can not remove this team event. You are not admin/owner of the Launchpad team or on the LoCo Council.'))
        return redirect( team_event_object )

@login_required
def team_event_select(request):
    teams = Team.objects.filter(lp_name__in=[g.name for g in request.user.groups.all()])
    if len(teams) == 0:
        request.user.message_set.create(message=_('You can not add a new team event. You are not member of any of the LoCo Teams.'))
        return redirect( 'event-list' )
    elif len(teams) == 1:
        from django.core import urlresolvers
        url = urlresolvers.reverse('team-event-new', args=[teams[0].lp_name])
        if request.GET.has_key('global_event_id'):
            return HttpResponseRedirect('%s?global_event_id=%s' % (url, request.GET.get('global_event_id', None)))
        else:
            return HttpResponseRedirect(url)
    else:
        context = {'teams': teams, 'global_event_id': request.GET.get('global_event_id', None)}
        return render_to_response('events/team_event_new_select.html', context, RequestContext(request))

def team_events_rss(request, team_slug):
    team_object = get_object_or_404(Team, lp_name=team_slug)
    events = TeamEvent.objects.filter(teams__lp_name=team_slug, date_end__gt=datetime.datetime.now()).order_by('date_begin')[:15]
    
    if request.META.has_key('wsgi.url_scheme'):
        scheme = request.META.get('wsgi.url_scheme')
    else:
        scheme = 'http'
        
    if request.META.has_key('HTTP_HOST'):
        host = request.META.get('HTTP_HOST')
    else:
        host = request.META.get('SERVER_NAME')
        
    if request.META.get('SERVER_PORT', 80) == 80 or request.META.get('SERVER_PORT', 80) == 0:
        base = '%s://%s' % (scheme, host)
    else:
        base = '%s://%s:%s' % (scheme, host, request.META.get('SERVER_PORT', 80))
    
    context = {'team_object': team_object, 'events': events, 'base':base}
    return render_to_response('events/team_events_rss.xml', context, RequestContext(request), mimetype='application/xhtml+xml')

@login_required
def team_event_new(request, team_slug):
    """
    new team event
    """
    team_object = get_object_or_404(Team, lp_name=team_slug)
    
    is_on_lc = launchpad.is_user_on_loco_council(request.user)
    is_member = launchpad.is_team_member(request.user, team_object)
    
    if is_on_lc or is_member:
        global_event = None
        if request.method == 'POST':
            form = TeamEventForm(data=request.POST)
            if form.is_valid():
                team_event = form.save()
                team_event.teams.add(team_object)
                request.user.message_set.create(message=_('New event created.'))
                return redirect( team_object )
        else:
            form = TeamEventForm(initial={'global_event':request.GET.get('global_event_id', None)})
            if request.GET.has_key('global_event_id'):
                global_event = get_object_or_404(GlobalEvent, pk=request.GET.get('global_event_id'))
                
        context = {
            'team_object': team_object,
            'form': form,
            'global_event': global_event,
        }
        return render_to_response('events/team_event_new.html', 
                                         context, RequestContext(request))
    else:
        # XXX: Once we move to a new ACL system, this needs fixing.
        request.user.message_set.create(message=_('You can not add a new event for this team. You are not member of the team or on the LoCo Council.'))
        return redirect( team_object )

@login_required
def team_event_copy(request, team_event_id):
    """
    new team event
    """
    
    team_event = get_object_or_404(TeamEvent, pk=team_event_id)
    team_object = team_event.teams.all()[0]
    team_event.id = team_event.pk = None

    is_on_lc = launchpad.is_user_on_loco_council(request.user)
    is_member = launchpad.is_team_member(request.user, team_object)
    
    if is_on_lc or is_member:
        if request.method == 'POST':
            form = TeamEventForm(instance=team_event, data=request.POST)
            if form.is_valid():
                team_event = form.save()
                team_event.teams.add(team_object)
                request.user.message_set.create(message=_('New event created.'))
                return redirect( team_object )
        else:
            form = TeamEventForm(instance=team_event)
        
        context = {
            'team_object': team_object,
            'form': form,
        }
        return render_to_response('events/team_event_new.html', 
                                         context, RequestContext(request))
    else:
        # XXX: Once we move to a new ACL system, this needs fixing.
        request.user.message_set.create(message=_('You can not add a new event for this team. You are not member of the team or on the LoCo Council.'))
        return redirect( team_object )

@login_required
def team_event_update(request, team_event_id):
    """
    update team event
    """
    team_event_object = get_object_or_404(TeamEvent, pk=team_event_id)
    #check if user is admin or owner of a team
    is_member = False
    for team in team_event_object.teams.all():
        if launchpad.is_team_member(request.user, team):
            is_member = True
            break
    
    is_on_lc = launchpad.is_user_on_loco_council(request.user)
    
    if is_on_lc or is_member:
        if request.method == 'POST':		
            form = TeamEventForm(data=request.POST, instance=team_event_object)
            if form.is_valid():
                form.save()
                request.user.message_set.create(message=_('Team Event updated.'))
                return redirect( team_event_object )
        else:
            form = TeamEventForm(instance=team_event_object)
            
        context = {
            'form': form,
        }
        return render_to_response('events/team_event_update.html', 
                                         context, RequestContext(request))
    else:
        request.user.message_set.create(message=_('You can not update this team event. You are not member of the team or on the LoCo Council.'))
        return redirect( team_event_object )

@login_required
def team_event_register(request, team_event_id):
    """
    register as attendee for a team event
    """
    team_event_object = get_object_or_404(TeamEvent, pk=team_event_id)
    if team_event_object.is_attending(request.user):
        attendee_object = team_event_object.attendee_set.get(attendee=request.user)
    else:
        attendee_object = Attendee(team_event=team_event_object, attendee=request.user)
        
    if request.method == 'POST':        
        form = AttendeeRegistrationForm(instance=attendee_object, data=request.POST)
        if form.is_valid():
            form.save()
            request.user.message_set.create(message=_('Your registration has been saved.'))
            return redirect( team_event_object )
    else:
        form = AttendeeRegistrationForm(instance=attendee_object, initial={'guests':0})
    
    context = {
           'form': form
    }
    
    return render_to_response('events/team_event_register.html', 
                                     context, RequestContext(request))


def team_event_comment_new(request, team_event_id):
    """
    create a comment for a team event
    """
    team_event_object = get_object_or_404(TeamEvent, pk=team_event_id)
    
    if request.method == 'POST':                
        form = TeamEventCommentForm(data=request.POST)
        if form.is_valid():
            team_event_comment = form.save(commit=False)
            team_event_comment.team_event = team_event_object
            team_event_comment.comment_by = request.user
            team_event_comment.save()
            request.user.message_set.create(message=_('Comment saved.'))
            return redirect( team_event_object )
    else:
        form = TeamEventCommentForm()
    
    context = {
        'team_event_object': team_event_object,
        'form': form,
    }
    return render_to_response('events/team_event_comment_new.html', 
                                     context, RequestContext(request))




#################################################################
# Global Events
#################################################################

def global_event_detail(request, global_event_id):
    """
    detailed view for a global event
    """
    global_event_object = get_object_or_404(GlobalEvent, pk=global_event_id)
    add_team_event = (request.user and request.user.is_authenticated() and \
                      request.user.groups.exclude(name__exact='ubuntu-lococouncil'))
    is_on_lc = launchpad.is_user_on_loco_council(request.user)
    context = {
        'global_event_object': global_event_object,
        'add_team_event': add_team_event,
	'user_is_on_lc': is_on_lc,
    }
    return render_to_response('events/global_event_detail.html', 
                                         context, RequestContext(request))

@login_required
def global_event_delete(request, global_event_id):
    """
    delete a event
    """
    global_event_object = get_object_or_404(GlobalEvent, pk=global_event_id)
    
    is_on_lc = launchpad.is_user_on_loco_council(request.user)
    
    if is_on_lc:
        if request.method == 'POST':		
            global_event_object.delete()
            request.user.message_set.create(message=_('Global Event removed.'))
            return redirect( 'event-list' )
        else:
            context = {'global_event_object': global_event_object}
            return render_to_response('events/global_event_delete_confirm.html', context, RequestContext(request))
    else:
        request.user.message_set.create(message=_('You can not remove this global event. You are not on the LoCo Council.'))
        return redirect( global_event_object )

@login_required
def global_event_new(request,):
    """
    new global event
    """
    is_on_lc = launchpad.is_user_on_loco_council(request.user)
    if is_on_lc:
        if request.method == 'POST':		
            form = GlobalEventForm(data=request.POST)
            if form.is_valid():
                form.save()
                request.user.message_set.create(message=_('New Global Event created.'))
                return redirect( 'event-list' )
        else:
            form = GlobalEventForm()
        
        context = {
            'form': form,
        }
        return render_to_response('events/global_event_new.html', 
                                         context, RequestContext(request))
    else:
        request.user.message_set.create(message=_('You can not add a new global event. You are not on the LoCo Council.'))
        return redirect( 'event-list' )

@login_required
def global_event_update(request, global_event_id):
    """
    update global event
    """
    global_event_object = get_object_or_404(GlobalEvent, pk=global_event_id)
    
    is_on_lc = launchpad.is_user_on_loco_council(request.user)
    if is_on_lc:
        if request.method == 'POST':		
            form = GlobalEventForm(data=request.POST, instance=global_event_object)
            if form.is_valid():
                form.save()
                request.user.message_set.create(message=_('Global Event updated.'))
                return redirect( global_event_object )
        else:
            form = GlobalEventForm(instance=global_event_object)
        
        context = {
            'form': form,
        }
        return render_to_response('events/global_event_update.html', 
                                         context, RequestContext(request))
    else:
        request.user.message_set.create(message=_('You can not update this global event. You are not on the LoCo Council.'))
        return redirect( global_event_object )




