from django.template import RequestContext
from django.shortcuts import render_to_response
from django.shortcuts import get_object_or_404
from django.contrib.auth.decorators import login_required
from django.http import HttpResponse, HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _
from django.utils.html import escape
from django.db.models import Q

from teams.models import Continent, countries_without_continent, countries_without_continent_have_venues
from models import Venue, venues_without_country
from forms import VenueForm, VenueSearchForm

from common.utils import simple_iterator

def venue_list(request):
    """
    a list with all venues
    """
    venue_list = Venue.objects.all()
    form = VenueSearchForm(data=request.GET)
    if form.is_valid():
        if form.cleaned_data['q']:
            q = form.cleaned_data['q']
            venue_list = venue_list.filter(Q(name__icontains=q) | Q(country__name__icontains=q) | Q(city__icontains=q) | Q(address__icontains=q))
    
    venue_list = venue_list.order_by('country')
    context = {
        'form': form,
        'venue_list': venue_list,
        'continents': Continent.objects.all().order_by('name'),
        'countries_without_continent': countries_without_continent().order_by('name'),
        'countries_without_continent_have_venues': countries_without_continent_have_venues(),
        'venues_without_country': venues_without_country().order_by('name'),
        'colcycle' : simple_iterator('col_left', 'col_right'),

    }
    return render_to_response('venues/venue_list.html', context,
                  RequestContext(request))

def venue_detail(request, venue_id):
    """
    detailed view for a venue
    """
    venue_object = get_object_or_404(Venue, pk=venue_id)
    context = {
        'venue_object': venue_object,
    }
    return render_to_response('venues/venue_detail.html', 
                                         context, RequestContext(request))

@login_required
def venue_new(request):
    """
    new venue
    """
    venue_object = Venue()
    if request.method == 'POST':		
        form = VenueForm(data=request.POST, instance=venue_object)
        if form.is_valid():
            form.save()
            request.user.message_set.create(message=_('New Venue created'))
            if request.REQUEST.has_key('_popup'):
                return HttpResponse('<script type="text/javascript">opener.dismissAddAnotherPopup(window, "%s", "%s");</script>' % \
                    # escape() calls force_unicode.
                    (escape(venue_object.pk), escape(venue_object)))
            else:
                return HttpResponseRedirect( reverse( 'venue-list' ) )
    else:
        form = VenueForm()
        
    context = {
        'form': form,
    }
    return render_to_response('venues/venue_update.html', 
                                     context, RequestContext(request))

@login_required
def venue_update(request, venue_id):
    """
 update venue
    """
    venue_object = get_object_or_404(Venue, pk=venue_id)
    
    if request.method == 'POST':		
        form = VenueForm(data=request.POST, instance=venue_object)
        if form.is_valid():
            form.save()
            request.user.message_set.create(message=_('Venue updated'))
            return HttpResponseRedirect( reverse( 'venue-detail', args=[venue_object.id] ) )
    else:
        form = VenueForm(instance=venue_object)
        
    context = {
        'form': form,
        'venue_object':venue_object,
    }
    return render_to_response('venues/venue_update.html', 
                                     context, RequestContext(request))

